@@ -17,12 +17,14 @@
 #' tree <- chelonia$phy
 #' dat <- chelonia$dat
 #' prior <- make.prior(tree)
-#' fit <- bayou.mcmc(tree, dat, model="OU", prior=prior, new.dir=TRUE, ngen=5000)
+#' fit <- bayou.mcmc(tree, dat, model="OU", prior=prior, 
+#'                                  new.dir=TRUE, ngen=5000)
 #' chain <- load.bayou(fit, save.Rdata=FALSE, cleanup=TRUE)
 #' plot(chain)
 #' }
 #' @export
-load.bayou <- function(bayouFit, save.Rdata=TRUE, file=NULL, cleanup=FALSE){#dir=NULL,outname="bayou",model="OU"){
+load.bayou <- function(bayouFit, save.Rdata=TRUE, file=NULL, 
+                       cleanup=FALSE){#dir=NULL,outname="bayou",model="OU"){
   tree <- bayouFit$tree
   dat <- bayouFit$dat
   outname <- bayouFit$outname
@@ -125,7 +127,8 @@ load.bayou <- function(bayouFit, save.Rdata=TRUE, file=NULL, cleanup=FALSE){#dir
 #' @param ... Optional arguments passed to \code{gelman.diag(...)} from the \code{coda} package
 #' 
 #' @export
-gelman.R <- function(parameter,chain1,chain2,freq=20,start=1,plot=TRUE, ...){
+gelman.R <- function(parameter,chain1,chain2,freq=20,start=1,
+                     plot=TRUE, ...){
   R <- NULL
   R.UCI <- NULL
   int <- seq(start,length(chain1[[parameter]]),freq)
@@ -229,12 +232,16 @@ Lposterior <- function(chain,tree,burnin=0, simpar=NULL,mag=TRUE){
 #' \dontrun{
 #' tree <- sim.bdtree(n=30)
 #' tree$edge.length <- tree$edge.length/max(branching.times(tree))
-#' prior <- make.prior(tree, dists=list(dk="cdpois", dsig2="dnorm", dtheta="dnorm"), 
-#'              param=list(dk=list(lambda=15, kmax=32), dsig2=list(mean=1, sd=0.01), 
-#'                  dtheta=list(mean=0, sd=3)), plot.prior=FALSE)
+#' prior <- make.prior(tree, dists=list(dk="cdpois", dsig2="dnorm", 
+#'              dtheta="dnorm"), 
+#'                param=list(dk=list(lambda=15, kmax=32), 
+#'                  dsig2=list(mean=1, sd=0.01), 
+#'                    dtheta=list(mean=0, sd=3)), 
+#'                      plot.prior=FALSE)
 #' pars <- priorSim(prior, tree, plot=FALSE, nsim=1)$pars[[1]]
 #' dat <- dataSim(pars, model="OU", phenogram=FALSE, tree)$dat
-#' fit <- bayou.mcmc(tree, dat, model="OU", prior=prior, new.dir=TRUE, ngen=5000, plot.freq=NULL)
+#' fit <- bayou.mcmc(tree, dat, model="OU", prior=prior, 
+#'              new.dir=TRUE, ngen=5000, plot.freq=NULL)
 #' chain <- load.bayou(fit, save.Rdata=TRUE, cleanup=TRUE)
 #' plotBayoupars(pull.pars(300, chain), tree)
 #' }
@@ -267,7 +274,13 @@ combine.chains <- function(chain1,chain2,burnin.prop=0){
   return(chains)
 }
 
-.buildControl <- function(pars, prior, move.weights=list("alpha"=4,"sig2"=2,"theta"=4, "slide"=2,"k"=10)){
+.buildControl <- function(pars, prior, move.weights=NULL){
+  model <- attributes(prior)$model
+  if(is.null(move.weights)){
+   move.weights <- switch(model, "OU"=list("alpha"=4,"sig2"=2,"theta"=4, "slide"=2,"k"=10),
+                                 "OUrepar" = list("halflife"=4, "Vy"=2, "theta"=4, "slide"=2, "k"=10),
+                                 "QG" = list("h2"=2, "P"=2, "w2"=3, "Ne"=3, "theta"=4, "slide"=2, "k"=10))
+  }
   ct <- unlist(move.weights)
   total.weight <- sum(ct)
   ct <- ct/sum(ct)
@@ -446,4 +459,33 @@ summary.bayouMCMC <- function(object, ...){
   out <- list(statistics=statistics, branch.posteriors=Lpost)
   invisible(out)
 }
-  
\ No newline at end of file
+
+#' Generate an overparameterized starting point for the MCMC
+#' 
+#' This function takes a prior function and generates a starting point that can be entered for \code{startpar} 
+#' in the function \code{bayou.mcmc}
+#' 
+#' @param prior A prior function
+#' @param tree A phylogenetic tree of class 'phylo'
+#' @param dat A named data vector
+#' 
+#' @details This function creates an "overparameterized" starting point for running the mcmc. It gives n-1 tips a unique
+#' optimum close to the actual data value. This is useful if you expect steep likelihood peaks that may be hard to find, 
+#' as these often will be easier to access from this overparameterized model. Generally, the overparameterization will have 
+#' a very high likelihood and a very low prior.
+overparameterize.startingPoint <- function(prior, tree, dat){
+  tree <- reorder(tree, "postorder")
+  dat <- dat[tree$tip.label]
+  model <- attributes(prior)$model
+  ntips <- length(tree$tip.label)
+  startpar <- priorSim(prior, tree, plot=FALSE, nsim=1)[[1]][[1]]
+  theta <- rnorm(ntips, dat, 1e-5)
+  startpar$theta <- theta
+  startpar$k <- ntips-1
+  startpar$sb <- which(tree$edge[,2] < ntips)
+  startpar$loc <- rep(0, startpar$k)
+  startpar$t2 <- 2:ntips
+  startpar$ntheta <- startpar$k+1
+  plotBayoupars(startpar, tree, col=setNames(rainbow(startpar$ntheta), 1:startpar$ntheta))
+  return(startpar)  
+}
\ No newline at end of file
